@@ -196,7 +196,7 @@ class EMRJobRunner(MRJobRunner):
     just for your job; it's also possible to run your job in an existing
     job flow by setting *emr_job_flow_id* (or :option:`--emr-job-flow-id`).
 
-    Input and support files can be either local or on S3; use ``s3://...``
+    Input, support, and jar files can be either local or on S3; use ``s3://...``
     URLs to refer to files on S3.
 
     This class has some useful utilities for talking directly to S3 and EMR,
@@ -256,8 +256,12 @@ class EMRJobRunner(MRJobRunner):
         :param emr_endpoint: optional host to connect to when communicating with S3 (e.g. ``us-west-1.elasticmapreduce.amazonaws.com``). Default is to infer this from *aws_region*.
         :type emr_job_flow_id: str
         :param emr_job_flow_id: the ID of a persistent EMR job flow to run jobs in (normally we launch our own job). It's fine for other jobs to be using the job flow; we give our job's steps a unique ID.
+        :type hadoop_streaming_jar: str
+        :param hadoop_streaming_jar: This is actually an option in the base MRJobRunner class. Points to a custom hadoop streaming jar on the local filesystem or S3. If you want to point to a streaming jar already installed on the EMR instances (perhaps through a bootstrap action?), use *hadoop_streaming_jar_on_emr*.
         :type num_ec2_instances: int
         :param num_ec2_instances: number of instances to start up. Default is ``1``.
+        :type hadoop_streaming_jar_on_emr: str
+        :param hadoop_streaming_jar_on_emr: Like *hadoop_streaming_jar*, except that it points to a path on the EMR instance, rather than to a local file or one on S3. Rarely necessary.
         :type s3_endpoint: str
         :param s3_endpoint: Host to connect to when communicating with S3 (e.g. ``s3-us-west-1.amazonaws.com``). Default is to infer this from *aws_region*.
         :type s3_log_uri: str
@@ -312,6 +316,11 @@ class EMRJobRunner(MRJobRunner):
             file_dict = self._add_bootstrap_file(path)
             self._bootstrap_python_packages.append(file_dict)
 
+        self._streaming_jar = None
+        if self._opts.get('hadoop_streaming_jar'):
+            self._streaming_jar = self._add_file_for_upload(
+                self._opts['hadoop_streaming_jar'])
+
         # if we're bootstrapping mrjob, keep track of the file_dict
         # for mrjob.tar.gz
         self._mrjob_tar_gz_file = None
@@ -349,15 +358,32 @@ class EMRJobRunner(MRJobRunner):
     def _allowed_opts(cls):
         """A list of which keyword args we can pass to __init__()"""
         return super(EMRJobRunner, cls)._allowed_opts() + [
-            'aws_access_key_id', 'aws_secret_access_key', 'aws_region',
-            'bootstrap_cmds', 'bootstrap_files', 'bootstrap_python_packages',
-            'bootstrap_scripts', 'check_emr_status_every', 'ec2_instance_type',
-            'ec2_key_pair', 'ec2_key_pair_file', 'ec2_master_instance_type',
-            'ec2_slave_instance_type', 'emr_endpoint', 'emr_job_flow_id',
-            'num_ec2_instances', 's3_endpoint', 's3_log_uri',
-            's3_scratch_uri', 's3_sync_wait_time', 'ssh_bin',
-            'ssh_bind_ports', 'ssh_tunnel_is_open',
-            'ssh_tunnel_to_job_tracker']
+            'aws_access_key_id',
+            'aws_secret_access_key',
+            'aws_region',
+            'bootstrap_cmds',
+            'bootstrap_files',
+            'bootstrap_python_packages',
+            'bootstrap_scripts',
+            'check_emr_status_every',
+            'ec2_instance_type',
+            'ec2_key_pair',
+            'ec2_key_pair_file',
+            'ec2_master_instance_type',
+            'ec2_slave_instance_type',
+            'emr_endpoint',
+            'emr_job_flow_id',
+            'hadoop_streaming_jar_on_emr',
+            'num_ec2_instances',
+            's3_endpoint',
+            's3_log_uri',
+            's3_scratch_uri',
+            's3_sync_wait_time',
+            'ssh_bin',
+            'ssh_bind_ports',
+            'ssh_tunnel_is_open',
+            'ssh_tunnel_to_job_tracker',
+        ]
 
     @classmethod
     def _default_opts(cls):
@@ -445,12 +471,6 @@ class EMRJobRunner(MRJobRunner):
 
         return s3_uri
 
-    def _add_bootstrap_file(self, path):
-        name, path = self._split_path(path)
-        file_dict = {'path': path, 'name': name, 'bootstrap': 'file'}
-        self._files.append(file_dict)
-        return file_dict
-
     def _run(self):
         self._setup_input()
         self._create_wrapper_script()
@@ -499,6 +519,12 @@ class EMRJobRunner(MRJobRunner):
 
             self._s3_input_uris.append(s3_input_dir)
 
+    def _add_bootstrap_file(self, path):
+        name, path = self._split_path(path)
+        file_dict = {'path': path, 'name': name, 'bootstrap': 'file'}
+        self._files.append(file_dict)
+        return file_dict
+
     def _setup_output(self):
         """Set self._output_dir if it's not set already."""
         if not self._output_dir:
@@ -741,6 +767,7 @@ class EMRJobRunner(MRJobRunner):
 
         log.info('Job flow created with ID: %s' % emr_job_flow_id)
         return emr_job_flow_id
+
     def _build_steps(self):
         """Return a list of boto Step objects corresponding to the
         steps we want to run."""
@@ -794,25 +821,27 @@ class EMRJobRunner(MRJobRunner):
             input = self._s3_step_input_uris(step_num)
             output = self._s3_step_output_uri(step_num)
 
-            # other arguments passed directly to hadoop streaming
-            step_args = []
-
-            for key, value in sorted(self._get_cmdenv().iteritems()):
-                step_args.extend(['-cmdenv', '%s=%s' % (key, value)])
-
-            for key, value in sorted(self._opts['jobconf'].iteritems()):
-                step_args.extend(['-jobconf', '%s=%s' % (key, value)])
-
-            step_args.extend(self._opts['hadoop_extra_args'])
+            step_args = self._hadoop_conf_args(step_num, len(steps))
 
             step_list.append(botoemr.StreamingStep(
                 name=name, mapper=mapper, reducer=reducer,
                 action_on_failure=action_on_failure,
                 cache_files=cache_files, cache_archives=cache_archives,
-                step_args=step_args, input=input, output=output))
+                step_args=step_args, input=input, output=output,
+                jar=self._get_jar()))
 
         return step_list
 
+    def _get_jar(self):
+        self._name_files()
+        self._pick_s3_uris_for_files()
+
+        if self._streaming_jar:
+            return self._streaming_jar['s3_uri']
+        else:
+            # this might be None, but that just means to use the default
+            return self._opts['hadoop_streaming_jar_on_emr']
+
     def _launch_emr_job(self):
         """Create an empty jobflow on EMR, and set self._emr_job_flow_id to
         the ID for that job."""
